<?php 

/*
Plugin Name: Buzer Gesetze verlinken
Plugin URI: http://www.buzer.de/li.htm
Description: verlinkt im Text zitierte Gesetzesverweise und EU-Normen
Author: www.buzer.de
Version: 1.0
Author URI: http://www.buzer.de/
*/


$buzer_config = array (
	# see also readme.txt

	# beliebige Anzahl Attribute zur Linkformatierung des Titels 
	# z.B. 'at'=>'target="_blank"' erzeugt Links wie <a href="ziel" target="_blank">...
	# kann beispielsweise auch fr die Formatierung (style, class, etc.) 
	# oder auch zum Plazieren von Skripten verwendet werden
	'at' 	=> '',
	
	# analog zu 'at' nur die verlinkten Paragraphen/Artikel betreffend
	'ap'	=> '',

	#  Zitierrichtung, Standand ist  1 der ZPO, d.h. der Titel wird rechts 
	# vom Paragraphen erwartet, dann ist hier keine Angabe notwendig, 
	# wird z.B. ZPO  1, 2, 3 etc. verwendet, funktioniert das Verlinken 
	# nur wenn das Flag 'lr' gesetz ist, z.B. 'lr' => 1
	'lr'	=> 0,
	
	# beeinflusst die Art der Verlinkung, fr Wordpress keine Angabe 
	# erforderlich, d.h. direkte Verlinkung per html ("<a href ...></a>")
	'type'	=> 0,

	# falls true, werden die Titel selbst nicht verlinkt, also 
	# z.B. bei  1 BGB wird nur der  1 verlinkt, allerdings 
	#erhlt dann natrlich auch "die Vorschriften des BGB bestimmen" keinen Link
	'not'	=> 0,
	
	# falls true, werden Sonderzeichen html-codiert
	'esc'	=> 1,
	
	# ersetzt aufeinanderfolgende Leerzeichen durch eines, 
	# fhrt zu besseren Ergebnissen, 
	# falls Text stark durch Verwendung von Leerzeichen formatiert ist, 
	# zerstrt dann allerdings auch diese Formatierung
	'trim'	=> 0,
	
	# fhrt ein Text lediglich Paragraphen auf aber nie den Titel selbst, 
	# weil klar ist, von welchem Titel die Rede ist,  wird auch keine Verlinkung 
	# erfolgen, weil das Plugin den Titel eben nicht kennt, hier hat man die Mglichkeit
	# einen Standardtitel anzugeben, z.B. 'dg'='EStG', wenn von eben diesem die 
	# Rede ist und im nheren Umfeld kein Titel gefunden wird 
	# ABER: da kaum in allen Artike immer die Rede vom EStG sein wird, 
	# macht eine Angabe hier im Plugin selten Sinn
	'dg'	=> '',
	
	# Zieladresse fr Postheader
	'_post'		=> '/li.htm',

	# Servername
	'_host'	=> 'www.buzer.de',
	
	# Port
	'_port'		=> 80,
	
	# timeout fr Verbindung zu buzer.de in Sekunden, falls berschritten 
	# wird der Eingangstext zurckgegeben
	'_timeout'	=> 5,
	
	# false hier eine Angabe erfolgt, wird eine Tabelle dieses Namens
	# als Cache verwendet
	# bei Angabe erfolgt die Verlinkung bei Anzeige des Artikels
	# wenn leer, wird der Artikel beim Speichern in der Datenbank verlinkt
	'_cache'	=> 'buzer_cache',
	
	# sollte wenn die Tabelle angelegt wurde, auf 1 gesetzt werden,
	# sonst wird jedesmal erst versucht die Tabelle zu erstellen
	'_cache_exists' => 0,
	
	# wenn dieser Text in der Seite auftaucht, stimmt etwas nicht denn 
	# es wurde die normale HTML-Seite von Buzer,de geladen
	# => Rckgabe Eingangstext
	'_skip'		=> 'DOCTYPE HTML',

	'charset'	=> get_option('blog_charset'),
	
	# Debug falls true
	'_debug'	=> 0,
	
);



if ( $buzer_config['_cache'] ) {
	# bei Anzeige, laden aus cache und ggf. vorher verlinken 
	add_filter('the_content', 'buzer_LinkIt');
	add_filter('comment_text', 'buzer_LinkIt');
} else {
	# nach jeder Bearbeitung neu verlinken und so speichern im Original Artikel
	add_filter('pre_comment_content', 'buzer_LinkIt');
	add_action('edit_post', 'buzer_LinkPostByID');
}


function buzer_createCacheTable( $table) {
	# legt falls erforderlich die cache-Tabelle an
	global $wpdb;
	
	$wpdb->query("create table if not exists " . $table . " (hash varchar(32) primary key, cache mediumtext not null, dateinsert datetime);");
	return $wpdb->get_row( "show tables like '$table'");

}

function buzer_getPostByID( $ID ) {
	# gibt den Text des Posts aus der Datenbank zurck
	global $wpdb;

	$sql = $wpdb->prepare( "select post_content from ".$wpdb->prefix."posts where id = %d limit 1;", $ID);
	$post = $wpdb->get_row( $sql)->post_content;
	return $post;
}

function buzer_updatePost( $ID, $Post ) {
	# gibt den Text des Posts aus der Datenbank zurck
	global $wpdb;

	$sql = $wpdb->prepare( "update ".$wpdb->prefix."posts set post_content=%s where id=%d limit 1;", $Post, $ID);
	if ( $wpdb->query( $sql)) {
		return 1;
	} else {
		return 0;
	}
}

function buzer_LinkPostByID( $ID) {
	$textin = buzer_getPostByID( $ID );
	if (!$textin) return $ID;
	$textout = buzer_LinkIt($textin);
	if ($textin == $textout) return $ID;
	buzer_updatePost( $ID, $textout );
	return $ID;
}

function buzer_getCache( $config, $text) {
	# gibt falls vorhandenen den cache zurck
	global $wpdb;

	$sql = $wpdb->prepare( "select cache from ".$config['_cache']." where hash = md5(%s) limit 1;", $text);
	$cache = $wpdb->get_row( $sql)->cache;
	return $cache;
	
}

function buzer_insertCache( $config, $text, $cache) {
	# schreibt die verlinkte version in den cache
	global $wpdb;

	$sql = $wpdb->prepare( "update ".$config['_cache']." set cache=%s, dateinsert=now() where hash=md5(%s) limit 1;", $cache, $text);
	if ( $wpdb->query( $sql)) {
		return 1;
	}
	
	$sql = $wpdb->prepare( "insert into ".$config['_cache']." (cache, hash, dateinsert) values (%s, md5(%s), now())", $cache, $text);
	return $wpdb->query( $sql );

}

function buzer_useCache( $config) {

	if ( !$config['_cache'] ) { return 0; }
	if ( !$config['_cache_exists'] and  !buzer_createCacheTable( $config['_cache']) ) { return 0; }
	$config['_cache_exists'] = 1;
	return 1;

}

function buzer_LinkIt($textin) {
	# verlinkt den bergebenen text
	global $buzer_config;
	
	if ( buzer_useCache( $buzer_config) and $cache=buzer_getCache( $buzer_config, $textin) ) { return $cache; }
	
	$content = 'plugin=1&site='.get_option('siteurl');
	
	foreach ($buzer_config as $key => $value) {
		if ( $value && !preg_match('/^_/', $key )) {
			$content .= "&$key=" . urlencode( $value );
		}
	}
	
	$content .= "&text=" . urlencode($textin);
	
	$header = "POST ".$buzer_config['_post']." HTTP/1.0\n";
	$header .= "Host: ".$buzer_config['_host']."\n";
	$header .= "Content-type: application/x-www-form-urlencoded; charset=" . $buzer_config['charset']."\n";
	$header .= "Content-length: " . strlen($content) . "\n\n";
	
	if ( $buzer_config['_debug']) echo "header=$header<br />";
	
	$socket = fsockopen($buzer_config['_host'], $buzer_config['_port'], $errno, $errstr, $buzer_config['_timeout']);
	
	if ( $buzer_config['_debug']) echo "fsockopen: socket=$socket, errno=$errno, errstr=$errstr<br />";
	
	if ( !$socket ) return $textin;
	
	if ( $buzer_config['_debug']) echo "Content: $content<br />";

	socket_set_timeout($socket, $buzer_config['_timeout'], 0);
	socket_set_blocking($socket, true);
	fputs($socket, $header.$content);
	$timed_out = false;
	$eof = false;
	$textout='';
	while (!$eof and !$timed_out) {
		$textout.= fgets($socket, 1024);
		$state = socket_get_status($socket);
		$eof = $state["eof"];
		$timed_out = $state["timed_out"];
	}
	fclose($socket);
	if ( $buzer_config['_debug']) echo "length: ".strlen($textout)."<br />";
	
	$del = "\n";
	if ( preg_match("/\r\n/", $textout) ) { $del="\r\n"; }
	
	$ok = false;
	$lines = explode($del, $textout);
	$i = 0;
	while ( $_ = $lines[$i++] ) {
		if ( preg_match('/200 OK/', $_) ) {
			$ok = true;
		}
	}
	
	if ( $buzer_config['_debug']) echo "read: ok=$ok, textout=$textout<br />";
	
	if ( !$ok ) {
		return $textin;
	}	
	
	$textout = implode($del, array_slice( $lines, $i));
	if ( $textin and strlen($textout)<strlen($textin) ) {
		if ( $buzer_config['_debug']) echo "Break zu kurz<br />";
		return $textin;
	} 
	if ( preg_match('/'.$buzer_config['_skip'].'/', $textout) ) {
		if ( $buzer_config['_debug']) echo "Break skip<br />";
		return $textin;
	} 
	
	if ( buzer_useCache( $buzer_config)) { buzer_insertCache( $buzer_config, $textin, $textout); }		
	return $textout;

}


?>